@@ -20,8 +20,7 @@ from tests import EVENT_TEMPLATE, TASK_TEMPLATE, VCARD_TEMPLATE
 
 import vdirsyncer.exceptions as exceptions
 from vdirsyncer.storage.base import Item
-from vdirsyncer.storage.dav import CaldavStorage, CarddavStorage, \
-    _normalize_href
+from vdirsyncer.storage.dav import CaldavStorage, CarddavStorage
 
 from .. import StorageTests, format_item
 
@@ -82,14 +81,14 @@ class TestCaldavStorage(DavStorageTests):
     def test_item_types_performance(self, get_storage_args, item_types,
                                     calls_num, monkeypatch, get_item):
         s = self.storage_class(item_types=item_types, **get_storage_args())
-        old_dav_query = s._dav_query
+        old_parse = s._parse_prop_responses
         calls = []
 
-        def _dav_query(*a, **kw):
+        def new_parse(*a, **kw):
             calls.append(None)
-            return old_dav_query(*a, **kw)
+            return old_parse(*a, **kw)
 
-        monkeypatch.setattr(s, '_dav_query', _dav_query)
+        monkeypatch.setattr(s, '_parse_prop_responses', new_parse)
         list(s.list())
         assert len(calls) == calls_num
 
@@ -183,26 +182,3 @@ class TestCarddavStorage(DavStorageTests):
     @pytest.fixture
     def item_template(self, request):
         return VCARD_TEMPLATE
-
-
-@pytest.mark.parametrize('base,path', [
-    ('http://example.com/', ''),
-    ('http://example.com/L%C3%98/', '/L%C3%98'),
-    ('http://example.com/LØ/', '/L%C3%98'),
-])
-def test_normalize_href(base, path):
-    assert _normalize_href(base, 'asdf') == path + '/asdf'
-
-    assert _normalize_href(base, 'hahah') == path + '/hahah'
-
-    assert _normalize_href(base, 'whoops@vdirsyncer.vcf') == \
-        path + '/whoops@vdirsyncer.vcf'
-
-    assert _normalize_href(base, 'whoops%40vdirsyncer.vcf') == \
-        path + '/whoops@vdirsyncer.vcf'
-
-    assert _normalize_href(base, 'wh%C3%98ops@vdirsyncer.vcf') == \
-        path + '/wh%C3%98ops@vdirsyncer.vcf'
-
-    assert _normalize_href(base, 'whØops@vdirsyncer.vcf') == \
-        path + '/wh%C3%98ops@vdirsyncer.vcf'
