@@ -23,21 +23,32 @@ dav_logger = log.get(__name__)
 CALDAV_DT_FORMAT = '%Y%m%dT%H%M%SZ'
 
 
-def _normalize_href(base, href, decoding_rounds=1):
+def _normalize_href(base, href):
     '''Normalize the href to be a path only relative to hostname and
     schema.'''
     if not href:
         raise ValueError(href)
     x = utils.urlparse.urljoin(base, href)
     x = utils.urlparse.urlsplit(x).path
-
-    for i in range(decoding_rounds):
-        x = utils.compat.urlunquote(x)
-
-    x = utils.compat.urlquote(x, '/@')
     return x
 
 
+def _encode_href(x):
+    return utils.compat.urlquote(x, '/@')
+
+
+def _decode_href(x):
+    return utils.compat.urlunquote(x)
+
+
+def _parse_xml(content):
+    try:
+        return etree.XML(content)
+    except etree.Error as e:
+        raise ValueError('Invalid XML encountered: {}\n'
+                         'Double-check the URLs in your config.'.format(e))
+
+
 class Discover(object):
     _resourcetype = None
     _homeset_xml = None
@@ -71,7 +82,7 @@ class Discover(object):
         """
         response = self.session.request('PROPFIND', '', headers=headers,
                                         data=body)
-        root = etree.XML(response.content)
+        root = _parse_xml(response.content)
 
         for element in root.iter('{*}current-user-principal'):
             for principal in element.iter():  # should be only one
@@ -90,8 +101,9 @@ class Discover(object):
         for principal in itertools.chain(self._find_principal(), ['']):
             for home in itertools.chain(self._find_home(principal), ['']):
                 for collection in self._find_collections(home):
-                    href = utils.urlparse.urljoin(self.session.url,
-                                                  collection['href'])
+                    collection['href'] = href = \
+                        utils.urlparse.urljoin(self.session.url,
+                                               collection['href'])
                     if href not in done:
                         done.add(href)
                         yield collection
@@ -117,7 +129,7 @@ class Discover(object):
         response = self.session.request('PROPFIND', home, headers=headers,
                                         data=self._collection_xml,
                                         is_subpath=False)
-        root = etree.XML(response.content)
+        root = _parse_xml(response.content)
         for response in root.iter('{*}response'):
             prop = response.find('{*}propstat/{*}prop')
             if prop.find('{*}resourcetype/{*}' + self._resourcetype) is None:
@@ -162,8 +174,7 @@ class DavSession(object):
     '''
 
     def __init__(self, url, username='', password='', verify=True, auth=None,
-                 useragent=USERAGENT, verify_fingerprint=None,
-                 dav_header=None):
+                 useragent=USERAGENT, verify_fingerprint=None):
         if username and not password:
             password = utils.get_password(username, url)
 
@@ -175,7 +186,6 @@ class DavSession(object):
         self.useragent = useragent
         self.url = url.rstrip('/') + '/'
         self.parsed_url = utils.urlparse.urlparse(self.url)
-        self.dav_header = dav_header
         self._session = None
 
     def request(self, method, path, data=None, headers=None,
@@ -186,23 +196,9 @@ class DavSession(object):
         assert url.startswith(self.url) or not is_subpath
         if self._session is None:
             self._session = requests_session()
-            self._check_dav_header()
         return utils.request(method, url, data=data, headers=headers,
                              session=self._session, **self._settings)
 
-    def _check_dav_header(self):
-        if self.dav_header is None:
-            return
-        headers = self.get_default_headers()
-        headers['Depth'] = 1
-        response = self.request(
-            'OPTIONS',
-            '',
-            headers=headers
-        )
-        if self.dav_header not in response.headers.get('DAV', ''):
-            raise ValueError('URL is not a collection')
-
     def get_default_headers(self):
         return {
             'User-Agent': self.useragent,
@@ -238,8 +234,6 @@ class DavStorage(Storage):
     fileext = None
     # mimetype of items
     item_mimetype = None
-    # The expected header for resource validation.
-    dav_header = None
     # XML to use when fetching multiple hrefs.
     get_multi_template = None
     # The LXML query for extracting results in get_multi
@@ -260,8 +254,7 @@ class DavStorage(Storage):
         if collection is not None:
             url = utils.urlparse.urljoin(url, collection)
         self.session = DavSession(url, username, password, verify, auth,
-                                  useragent, verify_fingerprint,
-                                  dav_header=self.dav_header)
+                                  useragent, verify_fingerprint)
         self.unsafe_href_chars = unsafe_href_chars
 
         # defined for _repr_attributes
@@ -276,8 +269,7 @@ class DavStorage(Storage):
             'username', 'password', 'verify', 'auth', 'useragent',
             'verify_fingerprint',
         ))
-        d = cls.discovery_class(DavSession(
-            url=url, dav_header=cls.dav_header, **discover_args))
+        d = cls.discovery_class(DavSession(url=url, **discover_args))
         for c in d.discover():
             base, collection = c['href'].rstrip('/').rsplit('/', 1)
             s = cls(url=base, collection=collection, **kwargs)
@@ -290,7 +282,7 @@ class DavStorage(Storage):
     def _get_href(self, item):
         href = item.ident
         for char in self.unsafe_href_chars:
-            href = item.ident.replace(char, '_')
+            href = href.replace(char, '_')
         return self._normalize_href(href + self.fileext)
 
     def get(self, href):
@@ -299,13 +291,15 @@ class DavStorage(Storage):
         return item, etag
 
     def get_multi(self, hrefs):
-        if not hrefs:
-            return ()
-        hrefs = [self._normalize_href(href) for href in hrefs]
-
+        hrefs = set(hrefs)
         href_xml = []
         for href in hrefs:
-            href_xml.append('<D:href>{}</D:href>'.format(href))
+            if href != self._normalize_href(href):
+                raise exceptions.NotFoundError(href)
+            href_xml.append('<D:href>{}</D:href>'.format(_encode_href(href)))
+        if not href_xml:
+            return ()
+
         data = self.get_multi_template.format(hrefs='\n'.join(href_xml))
         response = self.session.request(
             'REPORT',
@@ -313,25 +307,22 @@ class DavStorage(Storage):
             data=data,
             headers=self.session.get_default_headers()
         )
-        root = etree.XML(response.content)  # etree only can handle bytes
+        root = _parse_xml(response.content)  # etree only can handle bytes
         rv = []
         hrefs_left = set(hrefs)
-        for element in root.iter('{DAV:}response'):
-            href = self._normalize_href(
-                element.find('{DAV:}href').text)
-            raw = element \
-                .find('{DAV:}propstat') \
-                .find('{DAV:}prop') \
-                .find(self.get_multi_data_query).text
-            etag = element \
-                .find('{DAV:}propstat') \
-                .find('{DAV:}prop') \
-                .find('{DAV:}getetag').text
+        for href, etag, prop in self._parse_prop_responses(root):
+            try:
+                raw = prop.find(self.get_multi_data_query).text
+            except AttributeError:
+                dav_logger.warning('Skipping {}, the item content is missing.'
+                                   .format(href))
+                continue
+
             if isinstance(raw, bytes):
                 raw = raw.decode(response.encoding)
             if isinstance(etag, bytes):
                 etag = etag.decode(response.encoding)
-            rv.append((href, Item(raw), etag))
+
             try:
                 hrefs_left.remove(href)
             except KeyError:
@@ -341,6 +332,8 @@ class DavStorage(Storage):
                 else:
                     dav_logger.warning('Server sent unsolicited item: {}'
                                        .format(href))
+            else:
+                rv.append((href, Item(raw), etag))
         for href in hrefs_left:
             raise exceptions.NotFoundError(href)
         return rv
@@ -351,12 +344,11 @@ class DavStorage(Storage):
         if etag is None:
             headers['If-None-Match'] = '*'
         else:
-            assert etag[0] == etag[-1] == '"'
             headers['If-Match'] = etag
 
         response = self.session.request(
             'PUT',
-            href,
+            _encode_href(href),
             data=item.raw.encode('utf-8'),
             headers=headers
         )
@@ -380,59 +372,68 @@ class DavStorage(Storage):
     def delete(self, href, etag):
         href = self._normalize_href(href)
         headers = self.session.get_default_headers()
-        assert etag[0] == etag[-1] == '"'
         headers.update({
             'If-Match': etag
         })
 
         self.session.request(
             'DELETE',
-            href,
+            _encode_href(href),
             headers=headers
         )
 
-    def _dav_query(self, xml):
-        headers = self.session.get_default_headers()
+    def _parse_prop_responses(self, root, decoding_rounds=1):
+        hrefs = set()
+        for response in root.iter('{DAV:}response'):
+            try:
+                href = response.find('{DAV:}href').text
+            except AttributeError:
+                dav_logger.error('Skipping response, href is missing.')
+                continue
 
-        # CardDAV: The request MUST include a Depth header. The scope of the
-        # query is determined by the value of the Depth header. For example,
-        # to query all address object resources in an address book collection,
-        # the REPORT would use the address book collection as the Request- URI
-        # and specify a Depth of 1 or infinity.
-        # http://tools.ietf.org/html/rfc6352#section-8.6
-        #
-        # CalDAV:
-        # The request MAY include a Depth header.  If no Depth header is
-        # included, Depth:0 is assumed.
-        # http://tools.ietf.org/search/rfc4791#section-7.8
-        headers['Depth'] = 'infinity'
-        response = self.session.request(
-            'REPORT',
-            '',
-            data=xml,
-            headers=headers
-        )
-        root = etree.XML(response.content)
-        for element in root.iter('{DAV:}response'):
-            propstat = element.find('{DAV:}propstat')
-            prop = propstat.find('{DAV:}prop')
+            href = self._normalize_href(href)
+            for i in range(decoding_rounds):
+                href = _decode_href(href)
 
-            contenttype = prop.find('{DAV:}getcontenttype').text
-
-            href = self._normalize_href(element.find('{DAV:}href').text)
-            etag = prop.find('{DAV:}getetag').text
-            if not etag:
-                raise ValueError('Server did not return an etag for item {}. '
-                                 'Vdirsyncer is unable to function without '
-                                 'one.'.format(href))
-
-            if not contenttype or self.item_mimetype in contenttype:
-                yield href, etag
+            if href in hrefs:
+                # Servers that send duplicate hrefs:
+                # - Zimbra
+                #   https://github.com/untitaker/vdirsyncer/issues/88
+                # - Davmail
+                #   https://github.com/untitaker/vdirsyncer/issues/144
+                dav_logger.warning('Skipping identical href: {!r}'
+                                   .format(href))
+                continue
             else:
-                dav_logger.debug(
-                    'Skipping item with href {!r} '
-                    'because content type {!r} != {!r}.'
-                    .format(href, contenttype, self.item_mimetype))
+                hrefs.add(href)
+
+            try:
+                prop = response.find('{DAV:}propstat/{DAV:}prop')
+                contenttype = prop.find('{DAV:}getcontenttype')
+                etag = prop.find('{DAV:}getetag').text
+            except AttributeError:
+                dav_logger.warning('Skipping {!r}, properties are missing.'
+                                   .format(href))
+                continue
+
+            is_collection = prop.find('{DAV:}resourcetype/{DAV:}collection')
+            if is_collection is not None:
+                dav_logger.debug('Skipping {!r}, is collection.'.format(href))
+                continue
+
+            # different servers give different getcontenttypes:
+            # "text/vcard", "text/x-vcard", "text/x-vcard; charset=utf-8",
+            # "text/directory;profile=vCard", "text/directory",
+            # "text/vcard; charset=utf-8"
+            if self.item_mimetype is not None:
+                mediatype, subtype = self.item_mimetype.split('/')
+                if contenttype is not None and subtype not in contenttype.text:
+                    dav_logger.debug('Skipping {!r}, {!r} != {!r}.'
+                                     .format(href, contenttype,
+                                             self.item_mimetype))
+                    continue
+
+            yield href, etag, prop
 
 
 class CaldavStorage(DavStorage):
@@ -465,7 +466,6 @@ class CaldavStorage(DavStorage):
     storage_name = 'caldav'
     fileext = '.ics'
     item_mimetype = 'text/calendar'
-    dav_header = 'calendar-access'
     discovery_class = CalDiscover
 
     start_date = None
@@ -488,12 +488,14 @@ class CaldavStorage(DavStorage):
         super(CaldavStorage, self).__init__(**kwargs)
         if isinstance(item_types, str):
             orig_item_types = item_types
-            item_types = [x.strip() for x in item_types.split(',')]
+            item_types = list(filter(
+                bool, (x.strip() for x in item_types.split(','))
+            ))
 
             # XXX: Deprecation
             import json
             dav_logger.warning(
-                '{!r} is deprecated, please use:\nitem_types = {}'
+                '{!r} is deprecated, please use:\nitem_types = {}\n'
                 'The old form will be removed in 0.4.0.'
                 .format(orig_item_types, json.dumps(item_types)))
 
@@ -555,22 +557,29 @@ class CaldavStorage(DavStorage):
                 </C:filter>
             </C:calendar-query>'''
 
-        hrefs = set()
+        # CardDAV: The request MUST include a Depth header. The scope of the
+        # query is determined by the value of the Depth header. For example,
+        # to query all address object resources in an address book collection,
+        # the REPORT would use the address book collection as the Request- URI
+        # and specify a Depth of 1 or infinity.
+        # http://tools.ietf.org/html/rfc6352#section-8.6
+        #
+        # CalDAV: The request MAY include a Depth header.  If no Depth header
+        # is included, Depth:0 is assumed.
+        # http://tools.ietf.org/search/rfc4791#section-7.8
+        headers = self.session.get_default_headers()
+        headers['Depth'] = 'infinity'
 
         caldavfilters = self._get_list_filters(self.item_types,
                                                self.start_date, self.end_date)
 
         for caldavfilter in caldavfilters:
             xml = data.format(caldavfilter=caldavfilter)
-            for href, etag in self._dav_query(xml):
-                if href in hrefs:
-                    # Can't do stronger assertion here, see
-                    # https://github.com/untitaker/vdirsyncer/issues/88
-                    dav_logger.warning('Skipping identical href: {}'
-                                       .format(href))
-                    continue
-
-                hrefs.add(href)
+            response = self.session.request('REPORT', '', data=xml,
+                                            headers=headers)
+            root = _parse_xml(response.content)
+            rv = self._parse_prop_responses(root)
+            for href, etag, prop in rv:
                 yield href, etag
 
 
@@ -583,7 +592,6 @@ class CarddavStorage(DavStorage):
     storage_name = 'carddav'
     fileext = '.vcf'
     item_mimetype = 'text/vcard'
-    dav_header = 'addressbook'
     discovery_class = CardDiscover
 
     get_multi_template = '''<?xml version="1.0" encoding="utf-8" ?>
@@ -616,35 +624,10 @@ class CarddavStorage(DavStorage):
         # with Zimbra. See https://github.com/untitaker/vdirsyncer/issues/83
         response = self.session.request('PROPFIND', '', data=data,
                                         headers=headers)
+        root = _parse_xml(response.content)
 
-        root = etree.XML(response.content)
-        hrefs = set()
-        for element in root.iter('{DAV:}response'):
-            prop = element.find('{DAV:}propstat').find('{DAV:}prop')
-
-            resource_type = prop.find('{DAV:}resourcetype')
-            if resource_type.find('{DAV:}collection') is not None:
-                continue
-
-            content_type = prop.find('{DAV:}getcontenttype')
-            # different servers give different getcontenttypes:
-            # "text/vcard", "text/x-vcard", "text/x-vcard; charset=utf-8",
-            # "text/directory;profile=vCard", "text/directory",
-            # "text/vcard; charset=utf-8"
-            if 'vcard' not in content_type.text.lower():
-                continue
-
-            # Decode twice because ownCloud encodes twice.
-            # See https://github.com/owncloud/contacts/issues/581
-            href = self._normalize_href(element.find('{DAV:}href').text,
-                                        decoding_rounds=2)
-            etag = prop.find('{DAV:}getetag').text
-
-            if href in hrefs:
-                # Can't do stronger assertion here, see
-                # https://github.com/untitaker/vdirsyncer/issues/88
-                dav_logger.warning('Skipping identical href: {}'.format(href))
-                continue
-
-            hrefs.add(href)
+        # Decode twice because ownCloud encodes twice.
+        # See https://github.com/owncloud/contacts/issues/581
+        rv = self._parse_prop_responses(root, decoding_rounds=2)
+        for href, etag, prop in rv:
             yield href, etag
